﻿Imports System.Threading.Tasks

Namespace Tests

	''' <summary>
	''' A TaskScheduler to be used for unit testing. 
	''' The class allows to execute new scheduled tasks 
	''' on the same thread as a unit test.
	''' </summary>
	Public Class DeterministicTaskScheduler
		Inherits TaskScheduler
        Private ReadOnly scheduledTasks As New List(Of Task)

		#Region "TaskScheduler methods"
		Protected Overrides Sub QueueTask(ByVal task As Task)
			scheduledTasks.Add(task)
		End Sub
		Protected Overrides Function TryExecuteTaskInline(ByVal task As Task, ByVal taskWasPreviouslyQueued As Boolean) As Boolean
			scheduledTasks.Add(task)
			Return False
		End Function
		Protected Overrides Function GetScheduledTasks() As IEnumerable(Of Task)
			Return scheduledTasks
		End Function
        Public Overrides ReadOnly Property MaximumConcurrencyLevel As Integer
            Get
                Return 1
            End Get
        End Property
		#End Region

		''' <summary>
		''' Runs only the currently scheduled tasks.
		''' </summary>
		Public Sub RunPendingTasks()
            For Each task In scheduledTasks.ToArray
                TryExecuteTask(task)

                ' Propagate exceptions
                Try
                    task.Wait()
                Catch aggregateException As AggregateException
                    Throw aggregateException.InnerException
                Finally
                    scheduledTasks.Remove(task)
                End Try
            Next task
		End Sub

		''' <summary>
		''' Runs all tasks until no more scheduled tasks are left.
		''' If a pending task schedules an additional task it will also be executed.
		''' </summary>
		Public Sub RunTasksUntilIdle()
            Do While scheduledTasks.Any
                Me.RunPendingTasks()
            Loop
		End Sub

	End Class
End Namespace
